/*
 * Copyright (C) 2014 BlueKitchen GmbH
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Any redistribution, use, or modification is done solely for
 *    personal benefit and not for any commercial purpose or for
 *    monetary gain.
 *
 * THIS SOFTWARE IS PROVIDED BY BLUEKITCHEN GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL MATTHIAS
 * RINGWALD OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Please inquire about commercial licensing options at 
 * contact@bluekitchen-gmbh.com
 *
 */
#ifndef __SDP_H
#define __SDP_H

#include <stdint.h>
#include <btstack/linked_list.h>

#include "btstack-config.h"

#if defined __cplusplus
extern "C" {
#endif
    
typedef enum {
    SDP_Invalid = 0,
	SDP_ErrorResponse = 1,
	SDP_ServiceSearchRequest,
	SDP_ServiceSearchResponse,
	SDP_ServiceAttributeRequest,
	SDP_ServiceAttributeResponse,
	SDP_ServiceSearchAttributeRequest,
	SDP_ServiceSearchAttributeResponse
} SDP_PDU_ID_t;

// service record
// -- uses user_data field for actual
typedef struct {
    // linked list - assert: first field
    linked_item_t   item;
    
    // client connection
    void *  connection;
    
    // data is contained in same memory
    uint32_t        service_record_handle;
    uint8_t         service_record[1];  // waste 1 byte to allow compilation with older compilers
} service_record_item_t;

int sdp_handle_service_search_request(uint8_t * packet, uint16_t remote_mtu);
int sdp_handle_service_attribute_request(uint8_t * packet, uint16_t remote_mtu);
int sdp_handle_service_search_attribute_request(uint8_t * packet, uint16_t remote_mtu);


/* API_START */

/** 
 * @brief Set up SDP.
 */
void sdp_init(void);

void sdp_register_packet_handler(void (*handler)(void * connection, uint8_t packet_type, uint16_t channel, uint8_t *packet, uint16_t size));

#ifdef EMBEDDED
/** 
 * @brief Register service record internally - this version doesn't copy the record therefore it must be forever accessible. Preconditions:
    - AttributeIDs are in ascending order;
    - ServiceRecordHandle is first attribute and valid.
 * @return ServiceRecordHandle or 0 if registration failed.
 */
uint32_t sdp_register_service_internal(void *connection, service_record_item_t * record_item);
#endif

#ifndef EMBEDDED
/** 
 * @brief Register service record internally - this version creates a copy of the record precondition: AttributeIDs are in ascending order => ServiceRecordHandle is first attribute if present. 
 * @return ServiceRecordHandle or 0 if registration failed
 */
uint32_t sdp_register_service_internal(void *connection, uint8_t * service_record);
#endif

/** 
 * @brief Unregister service record internally.
 */
void sdp_unregister_service_internal(void *connection, uint32_t service_record_handle);
/* API_END */

#if defined __cplusplus
}
#endif
#endif // __SDP_H
